class ActiveStix::CampaignsController < ApplicationController
  before_action :set_stix_campaign, only: [:show, :edit, :update, :destroy]
  before_action :set_threat_actor

  # GET /stix/campaigns
  # GET /stix/campaigns.json
  def index
    @campaigns = ActiveStix::Campaign.all
  end

  # GET /stix/campaigns/1
  # GET /stix/campaigns/1.json
  def show
  end

  # GET /stix/campaigns/new
  def new
    @campaign = ActiveStix::Campaign.new
  end

  # GET /stix/campaigns/1/edit
  def edit
  end

  # POST /stix/campaigns
  # POST /stix/campaigns.json
  def create
    @campaign = ActiveStix::Campaign.new(stix_campaign_params)

    respond_to do |format|
      if @campaign.save
        ActiveStix::Relationship.relate(@campaign, @threat_actor, "attributed-to")
        format.html { redirect_to [@threat_actor, @campaign], notice: 'Campaign was successfully created.' }
        format.json { render :show, status: :created, location: @campaign }
      else
        format.html { render :new }
        format.json { render json: @campaign.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH/PUT /stix/campaigns/1
  # PATCH/PUT /stix/campaigns/1.json
  def update
    respond_to do |format|
      if @campaign.update(stix_campaign_params)
        format.html { redirect_to @campaign, notice: 'Campaign was successfully updated.' }
        format.json { render :show, status: :ok, location: @campaign }
      else
        format.html { render :edit }
        format.json { render json: @campaign.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /stix/campaigns/1
  # DELETE /stix/campaigns/1.json
  def destroy
    @campaign.destroy
    respond_to do |format|
      format.html { redirect_to stix_campaigns_url, notice: 'Campaign was successfully destroyed.' }
      format.json { head :no_content }
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_stix_campaign
      @campaign = ActiveStix::Campaign.find(params[:id])
    end

    def set_threat_actor
      @threat_actor = ActiveStix::ThreatActor.find(params[:threat_actor_id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def stix_campaign_params
      params.require(:campaign).permit(:name, :description, :first_seen, :last_seen, :objective)
    end
end
