class ActiveStix::UserAccountsController < ApplicationController
  before_action :set_stix_user_account, only: [:show, :edit, :update, :destroy]

  # GET /stix/user_accounts
  # GET /stix/user_accounts.json
  def index
    @user_accounts = ActiveStix::UserAccount.all
  end

  # GET /stix/user_accounts/1
  # GET /stix/user_accounts/1.json
  def show
  end

  # GET /stix/user_accounts/new
  def new
    @user_account = ActiveStix::UserAccount.new
  end

  # GET /stix/user_accounts/1/edit
  def edit
  end

  # POST /stix/user_accounts
  # POST /stix/user_accounts.json
  def create
    @user_account = ActiveStix::UserAccount.new(stix_user_account_params)

    respond_to do |format|
      if @user_account.save
        format.html { redirect_to @user_account, notice: 'User account was successfully created.' }
        format.json { render :show, status: :created, location: @user_account }
      else
        format.html { render :new }
        format.json { render json: @user_account.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH/PUT /stix/user_accounts/1
  # PATCH/PUT /stix/user_accounts/1.json
  def update
    respond_to do |format|
      if @user_account.update(stix_user_account_params)
        format.html { redirect_to @user_account, notice: 'User account was successfully updated.' }
        format.json { render :show, status: :ok, location: @user_account }
      else
        format.html { render :edit }
        format.json { render json: @user_account.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /stix/user_accounts/1
  # DELETE /stix/user_accounts/1.json
  def destroy
    @user_account.destroy
    respond_to do |format|
      format.html { redirect_to stix_user_accounts_url, notice: 'User account was successfully destroyed.' }
      format.json { head :no_content }
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_stix_user_account
      @user_account = ActiveStix::UserAccount.find(params[:id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def stix_user_account_params
      params.require(:user_account).permit(:user_id, :account_login, :account_type, :display_name, :is_service_account, :is_privileged, :can_escalate_privs, :is_disabled, :account_created, :account_expires, :password_last_changed, :account_first_login, :account_last_login)
    end
end
