class ActiveStix::Campaign < ApplicationRecord
  has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
  has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target

  def self.ingest_json(obj)
    camp = find_or_create_by(stix_id: obj['id'], name: obj['name'], description: obj['description'], first_seen: obj['first_seen'], last_seen: obj['last_seen'])
    camp
  end

  before_create do
    self.stix_id = "#{type}--#{SecureRandom.uuid}" if stix_id.blank?
  end


  def type
    'campaign'
  end


  def attack_patterns
    attack_pattern_relationships.map(&:target)
  end

  def attack_pattern_relationships
    relationships = []
    ActiveStix::Relationship.where(source: self, relationship_type: "uses", target_type: "ActiveStix::AttackPattern").each do |rel|
      relationships << rel
    end
    relationships
  end

  def indicator_relationships
    relationships = []
    ActiveStix::Relationship.where(source_type: "ActiveStix::Indicator", relationship_type: "indicates", target: self).each do |rel|
      relationships << rel
    end
    relationships
  end

  def as_stix
    as_json(only: []).tap do |hash|
      hash["id"] = stix_id
      hash["type"] = type
      hash["created"] = created_at.rfc3339(3)
      hash["modified"] = updated_at.rfc3339(3)
      hash["first_seen"] = created_at.rfc3339(3)
      hash["last_seen"] = updated_at.rfc3339(3)
      hash["name"] = name
      hash["description"] = description
    end
  end

  def convert_to_json
    {
        :type => "campaign",
        :id => stix_id,
        :created => created_at.to_s,
        :modified => updated_at.to_s,
        :name => name,
        :description => description,
        :first_seen => first_seen,
        :last_seen => last_seen
    }
  end
end
