module ActiveStix
  class CourseOfAction < ApplicationRecord
    has_many :reference_items, as: 'referrer'
    has_many :external_references, class_name: 'ActiveStix::ExternalReference', through: :reference_items

    has_many :reference_object_marking_courses, class_name: 'ActiveStix::ReferenceObjectMarkingCourse', foreign_key: 'stix_course_of_action_id'
    has_many :marking_definitions, class_name: 'ActiveStix::MarkingDefinition', through: :reference_object_marking_courses, primary_key: 'id'

    has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
    has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target

    def self.ingest_json(obj)
      course_of_action = find_or_create_by(stix_id: obj['id'], name: obj['name'], description: obj['description'])
      if obj.has_key?('external_references')
        obj['external_references'].each do |er|
          external_reference = ExternalReference.ingest_json(er, obj['id'])
          course_of_action.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference:external_reference, referrer: course_of_action)
        end
      end
      if obj.has_key?('object_marking_refs')
        # TODO
        # obj['object_marking_refs'].each do |mr|
        #   marking_definition = MarkingDefinition.create_by_id(mr)
        #   course_of_action.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingCourse.find_by(stix_marking_definition_id:marking_definition.id, stix_course_of_action_id:course_of_action.id)
        # end
      end
      course_of_action.save
      course_of_action
    end

    def convert_to_json
      external_refs_arr = []
      external_references.each do | x |
        external_refs_arr << x.convert_to_json
      end

      marking_def_arr = []
      marking_definitions.each do | x |
        marking_def_arr << x.convert_to_json
      end

      {
          :external_references => external_refs_arr,
          :object_marking_refs => marking_def_arr,
          :id => stix_id,
          :name => name,
          :type => "course-of-action",
          :modified => updated_at.to_s,
          :created => created_at.to_s,
          :description => description
      }
    end
  end
end
