class ActiveStix::Indicator < ApplicationRecord
  has_many :active_stix_indicator_labels, :class_name => 'ActiveStix::IndicatorLabel', foreign_key: 'stix_indicator_id'
  has_many :markups, as: :labelable
  has_many :labels, through: :markups

  before_create do
    self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
  end


  def self.ingest_json(obj)
    indicator = find_or_create_by(stix_id:obj['id'], name:obj['name'], description:obj['description'], pattern:obj['pattern'], valid_from:obj['valid_from'])

    if obj.has_key?('labels')
      obj['labels'].each do | lab |
        label = ActiveStix::Label.ingest_label('indicator-labels', lab)
        indicator.labels << label unless ActiveStix::Markup.find_by(labelable: indicator, label: label)
      end
    end

    indicator.save
    indicator
  end


  def attack_pattern_relationships
    relationships = []
    ActiveStix::Relationship.where(source: self, relationship_type: "uses", target_type: "ActiveStix::AttackPattern").or(
        ActiveStix::Relationship.where(source: self, relationship_type: "indicates", target_type: "ActiveStix::AttackPattern")
    ).each do |rel|
      relationships << rel
    end
    relationships
  end

  def attack_patterns
    attack_pattern_relationships.collect {|rel| rel.target}
  end



  def type
    'indicator'
  end

  def as_stix(classification = nil, chess = nil)
    as_json(only: [:name, :description, :pattern]).tap do |hash|
      hash["id"] = stix_id
      hash["type"] = type
      hash["description"] = description
      hash["created"] = created_at.rfc3339(3)
      hash["labels"] = ["malicious-activity"]
      hash["valid_from"] = (valid_from || updated_at).rfc3339(3)
      hash["modified"] = updated_at.rfc3339(3)
    end
  end

end
