class ActiveStix::IntrusionSet < ApplicationRecord
  has_many :reference_items, as: 'referrer'
  has_many :external_references, class_name: 'ActiveStix::ExternalReference', through: :reference_items
  # has_many :reference_object_marking_intrusion_set, class_name: 'ActiveStix::ReferenceObjectMarkingIntrusionSet', foreign_key: 'stix_intrusion_set_id' todo
  # has_many :marking_definitions, class_name: 'ActiveStix::MarkingDefinition', through: :reference_object_marking_intrusion_set todo
  has_many :reference_object_intrusion_alias, class_name: 'ActiveStix::ReferenceObjectIntrusionAlias', foreign_key: 'stix_intrusion_set_id'
  has_many :intrusion_set_aliases, class_name: 'ActiveStix::IntrusionSetAlias', through: :reference_object_intrusion_alias
  has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
  has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target


  before_create do
    self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
  end

  def type
    'intrusion-set'
  end


  def self.ingest_json(obj)
    intrusion_set = find_or_create_by(stix_id: obj['id'], name: obj['name'])

    if obj.has_key?('description')
      intrusion_set.description = obj['description']
    end

    if obj.has_key?('external_references')
      obj['external_references'].each do |er|
        external_reference = ActiveStix::ExternalReference.ingest_json(er, obj['id'])
        intrusion_set.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference: external_reference, referrer: intrusion_set)
      end
    end

    if obj.has_key?('object_marking_refs')
      # obj['object_marking_refs'].each do |mr|
      #   marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
      #   intrusion_set.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingIntrusionSet.find_by(stix_marking_definition_id: marking_definition.id, stix_intrusion_set_id: intrusion_set.id)
      # end todo
    end

    if obj.has_key?('aliases')
        intrusion_set.aliases = obj['aliases']
    end
    intrusion_set.save
    intrusion_set
  end

  def as_stix(classification = nil, chess = nil)
    as_json(only: [:name]).tap do |hash|
      hash["type"] = type
      hash["description"] = description if description
      hash["created"] = created_at.utc.iso8601(3)
      hash["modified"] = updated_at.utc.iso8601(3)
      hash["id"] = stix_id
      hash["spec_version"] = "2.0"
    end
  end

  def convert_to_json
    {
        :type => "intrusion-set",
        :name => name,
        :id => stix_id,
        :created => created_at.to_s,
        :modified => updated_at.to_s
    }
  end
end
