class ActiveStix::Phase < ApplicationRecord
  belongs_to :kill_chain
  has_many :kill_chain_phases
  has_many :attack_patterns, class_name: 'ActiveStix::AttackPattern', through: :kill_chain_phases, foreign_key: 'attack_pattern_ref', primary_key: 'stix_id'

  @@rank_map = {
      'initial-access' => 0,
      'execution' => 1,
      'persistence' => 2,
      'privilege-escalation' => 3,
      'defense-evasion' => 4,
      'credential-access' => 5,
      'discovery' => 6,
      'lateral-movement' => 7,
      'collection' => 8,
      'command-and-control' => 9,
      'exfiltration' => 10,
      'impact' => 11
  }

  def self.ingest_json(obj)
    kill_chain = ActiveStix::KillChain.find_or_create_by(name: obj['kill_chain_name'])
    phase = kill_chain.phases.find_or_create_by(name: obj['phase_name'], rank:@@rank_map[obj['phase_name']])
    phase
  end

  def as_stix
    as_json(only: []).tap do |hash|
      hash["phase_name"] = name
      hash["kill_chain_name"] = kill_chain.name
    end
  end

  def convert_to_json
    {
        :kill_chain_name => kill_chain.convert_to_json,
        :phase_name => name
    }
  end

end
