class ActiveStix::Relationship < ApplicationRecord
  belongs_to :source, :polymorphic => true, foreign_key: 'source_ref', primary_key: 'stix_id'
  belongs_to :target, :polymorphic => true, foreign_key: 'target_ref', primary_key: 'stix_id'
  #has_many :marking_definitions, :class_name => 'ActiveStix::MarkingDefinition', foreign_key: 'relationship_ref', primary_key: 'stix_id'
  #has_many :object_markings, :class_name => 'ActiveStix::ObjectMarking', foreign_key: 'stix_relationship_id', primary_key: 'stix_id'

  has_many :reference_object_marking_relationships, class_name: 'ActiveStix::ReferenceObjectMarkingRelationship', foreign_key: 'stix_relationship_id'
  has_many :marking_definitions, class_name: 'ActiveStix::MarkingDefinition', through: :reference_object_marking_relationships


  before_create do
    self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
  end

  @@stix_map = {
      'bundle' => ActiveStix::Bundle,
      'attack-pattern' => ActiveStix::AttackPattern,
      'relationship' => ActiveStix::Relationship,
      'course-of-action' => ActiveStix::CourseOfAction,
      'identity' => ActiveStix::Identity,
      'intrusion-set' => ActiveStix::IntrusionSet,
      'malware' => ActiveStix::Malware,
      'tool' => ActiveStix::Tool,
      'marking-definition' => ActiveStix::MarkingDefinition,
      'report' => ActiveStix::Report,
      'campaign' => ActiveStix::Campaign,
      'indicator' => ActiveStix::Indicator
  }

  def self.ingest_json(obj)
    #puts "Loc B: #{obj['id'].to_s}"
    #puts "source: #{obj['source_ref']}"
    #puts "target: #{obj['target_ref']}"
    #pause = gets

    src_type = obj['source_ref'].split('--')[0]
    tgt_type = obj['target_ref'].split('--')[0]

    src = @@stix_map[src_type].where("stix_id = ?", obj['source_ref']).first
    tgt = @@stix_map[tgt_type].where("stix_id = ?", obj['target_ref']).first


    relationship = find_or_create_by!(stix_id: obj['id'], source: src, target: tgt, relationship_type: obj['relationship_type'], source_type: @@stix_map[src_type].to_s, target_type: @@stix_map[tgt_type].to_s)

    if obj.has_key?('description')
      relationship.description = obj['description']
    end

    if obj.has_key?('object_marking_refs')
      obj['object_marking_refs'].each do |mr|
        # marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
        # relationship.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingRelationship.find_by(stix_marking_definition_id: marking_definition.id, stix_relationship_id: relationship.id)
        # todo
      end
    end

    relationship.save
    relationship
  end

  def self.relate(source, target, relationship_type)
    find_or_create_by!(source: source, target: target, relationship_type: relationship_type)
  end

  def type
    'relationship'
  end

  def convert_to_json
    {
        :type => "relationship",
        :id => stix_id,
        :created => created_at.to_s,
        :modified => updated_at.to_s,
        :relationship_type => relationship_type,
        :source_ref => source_ref.to_s,
        :target_ref => target_ref.to_s,
        :description => description
    }
  end

  def as_stix

    as_json(only: [:relationship_type]).tap do |hash|
      hash["id"] = stix_id
      hash["description"] = description if description
      hash["type"] = type
      hash["created"] = created_at.utc.iso8601(3)
      hash["modified"] = created_at.utc.iso8601(3)
      hash["source_ref"] = source.stix_id
      hash["target_ref"] = target.stix_id
      hash["spec_version"] = "2.0"
    end
  end
end
