
class ActiveStix::Report < ApplicationRecord

  has_many :report_objects, primary_key: 'stix_id', foreign_key: 'report_ref'

  has_many :markups, as: :labelable
  has_many :labels, through: :markups

  before_create do
    self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
  end


  @@stix_map = {
      'bundle' => ActiveStix::Bundle,
      'attack-pattern' => ActiveStix::AttackPattern,
      'relationship' => ActiveStix::Relationship,
      'course-of-action' => ActiveStix::CourseOfAction,
      'identity' => ActiveStix::Identity,
      'intrusion-set' => ActiveStix::IntrusionSet,
      'malware' => ActiveStix::Malware,
      'tool' => ActiveStix::Tool,
      'marking-definition' => ActiveStix::MarkingDefinition,
      'report' => ActiveStix::Report,
      'campaign' => ActiveStix::Campaign,
      'indicator' => ActiveStix::Indicator
  }

  def type
    'report'
  end

  def self.ingest_json(obj)
    report = find_or_create_by(stix_id:obj['id'], name:obj['name'], description:obj['description'], published:obj['published'])
    if obj.has_key?('labels')
      obj['labels'].each do | lab |
        label = ActiveStix::Label.ingest_label('report-labels', lab)
        report.labels << label unless ActiveStix::Markup.find_by(labelable: report, label: label)
      end
    end
    report.save
    report
  end


  def self.add_obj_refs(report_json)
    report = find_by(stix_id: report_json['id'], name: report_json['name'])
    if report_json.has_key?('object_refs')
      report_json['object_refs'].each do |obj_refs|
        report.add(obj_refs)
      end
    end
    report.save
  end


  def add(stix_object)
    unless includes?(stix_object)
      #report_object = Bundle.find_or_create_object(stix_object) # todo using object prefix and stix map
      #report_objects << report_object
      #report_objects.create(stix_object_ref: stix_object, stix_object_type: obj_type)
      obj_type_str = stix_object.split('--')[0]
      #obj_type = @@stix_map[obj_type_str]
      obj = @@stix_map[obj_type_str].where("stix_id = ?", stix_object).first
      report_objects.create(stix_object: obj) unless report_objects.find_by(report_ref: stix_id, object_ref: obj)
    end
  end

  #used by phishing report
  def add_stix_object(stix_object)
    unless includes?(stix_object)
      report_objects.create(stix_object: stix_object)
    end
  end

  def includes?(stix_object)
    report_objects.where(object_ref: stix_object).any?
  end

  def as_stix(classification = nil, chess = nil)
    as_json(only:[:name, :description]).tap do |hash|
      hash["id"] = stix_id
      hash["type"] = type
      hash["description"] = description 
      hash["created"] = created_at.rfc3339(3)
      hash["modified"] = updated_at.rfc3339(3)
      hash["published"] = (published || updated_at).rfc3339(3)
      hash["object_refs"] = report_objects.collect{|ro| ro.stix_object.stix_id}
      hash["labels"] = labels.collect{|label| label.name}
    end
  end

end
