class ActiveStix::Tool < ApplicationRecord
  has_many :reference_items, as: 'referrer'
  has_many :external_references, through: :reference_items

  has_many :markups, as: :labelable
  has_many :labels, through: :markups

  def self.expected_keys
    [
        'description',
        'external_references',
        'object_marking_refs',
        'created_by_ref',
        'labels'
    ]
  end

  def type
    'tool'
  end

  def self.ingest_json(obj)
    tool = find_or_create_by(stix_id: obj['id'], name: obj['name'])

    expected_keys.each do |expected_key|
      if obj.has_key?(expected_key)
        send(expected_key, *[tool, obj])
      end
    end

    tool.save
    tool
  end

  def self.object_marking_refs(tool, obj)
    # obj['object_marking_refs'].each do |mr|
    #   marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
    #   tool.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingTool.find_by(stix_marking_definition_id:marking_definition.id, stix_tool_id:tool.id)
    # end todo
  end

  def self.description(tool, obj)
    tool.description = obj['description']
  end

  def self.created_by_ref(tool, obj)
    tool.created_by_ref = obj['created_by_ref']
  end

  def self.external_references(tool, obj)
    obj['external_references'].each do |er|
      external_reference = ActiveStix::ExternalReference.ingest_json(er, obj['id'])
      tool.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference: external_reference, referrer: tool)
    end
  end


  def self.labels(tool, obj)
    obj['labels'].each do |lab|
      label = ActiveStix::Label.ingest_label('tool', lab)
      tool.labels << label unless ActiveStix::Markup.find_by(labelable: tool, label: label)
    end
  end


  def as_stix(classification = nil, chess = nil)
    as_json
  end


  def convert_to_json
    external_refs_arr = []
    external_references.each do |x|
      external_refs_arr << x.convert_to_json
    end

    # marking_def_arr = []
    # marking_definitions.each do | x |
    #   marking_def_arr << x.convert_to_json
    # end todo

    platform_arr = []
    platforms.each do |x|
      platform_arr << x.convert_to_json
    end

    labels_arr = []
    labels.each do |x|
      labels_arr << x.open_vocabulary.convert_to_json
    end


    alias_arr = []
    attack_aliases.each do |x|
      alias_arr << x.convert_to_json
    end

    {
        :external_references => external_refs_arr,
        # :object_marking_refs => marking_def_arr,
        :modified => updated_at.to_s,
        :created_by_ref => created_by_ref,
        :id => stix_id,
        :name => name,
        :created => created_at.to_s,
        :labels => labels_arr,
        :type => "tool",
        :description => description
    }
  end

end
