class ActiveStix::Malware < ApplicationRecord
  has_many :reference_items, as: 'referrer'
  has_many :external_references, class_name: 'ActiveStix::ExternalReference', through: :reference_items

  has_many :reference_object_marking_malware, class_name: 'ActiveStix::ReferenceObjectMarkingMalware', foreign_key: 'stix_malware_id'
  has_many :marking_definitions, class_name: 'ActiveStix::MarkingDefinition', through: :reference_object_marking_malware

  has_many :markups, as: :labelable
  has_many :labels, through: :markups

  def self.expected_keys
    [
        'description',
        'external_references',
        'object_marking_refs'
    ]
  end

  def type
    'malware'
  end

  def self.ingest_json(obj)
    malware = find_or_create_by(stix_id: obj['id'], name: obj['name'], created_by_ref: obj['created_by_ref'])

    expected_keys.each do |expected_key|
      if obj.has_key?(expected_key)
        send(expected_key, *[malware, obj])
      end
    end

    malware.save
    malware
  end

  def self.description(malware, obj)
    malware.description = obj['description']
  end

  def self.external_references(malware, obj)
    obj['external_references'].each do |er|
      external_reference = ActiveStix::ExternalReference.ingest_json(er, obj['id'])
      malware.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference: external_reference, referrer: malware)
    end
  end

  def self.object_marking_refs(malware, obj)
    obj['object_marking_refs'].each do |mr|
      # todo
      # marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
      # malware.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingMalware.find_by(stix_marking_definition_id:marking_definition.id, stix_malware_id:malware.id)
    end
  end

  def as_stix(classification = nil, chess = nil)
    as_json
  end

  def convert_to_json
    external_refs_arr = []
    external_references.each do |x|
      external_refs_arr << x.convert_to_json
    end

    marking_def_arr = []
    marking_definitions.each do |x|
      marking_def_arr << x.convert_to_json
    end

    platform_arr = []
    platforms.each do |x|
      platform_arr << x.convert_to_json
    end

    labels_arr = []
    labels.each do |x|
      labels_arr << x.open_vocabulary.convert_to_json
    end

    alias_arr = []
    attack_aliases.each do |x|
      alias_arr << x.convert_to_json
    end

    {
        :external_references => external_refs_arr,
        :object_marking_refs => marking_def_arr,
        :modified => updated_at.to_s,
        :created_by_ref => created_by_ref,
        :id => stix_id,
        :name => name,
        :created => created_at.to_s,
        :labels => labels_arr,
        :type => "malware",
        :description => description
    }
  end
end
